#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

#This is used to backup ldap server data. It should be put into
# /opt/ibm/director/vmi/upgrade/script/05-save-ldap-config.sh
# log file is in /spare/ldap/05-save-ldap-config.log
# Code added for HA handling (chrg command).

SAVE_LDAP_BACKUP_DIR=/spare/ldap/backup.ldif
SAVE_LDAP_LOG=/spare/ldap/05-save-ldap-config.log
PID_FILE=/var/run/slapd/slapd.pid

source /opt/ibm/director/lwi/runtime/itme/eclipse/plugins/bin/RGFunctions.sh
mkdir -p /spare/ldap

echo "********Start ldap upgrade at `date` ********" >$SAVE_LDAP_LOG
#Check for HA status
#Exit Status for chrg command
# 0 Command has run successfully.
# 1 Error occurred with RMC. 
# 2 Error occurred with CLI script. 
# 3 Incorrect flag on command line.
# Run get HA status
/opt/ibm/director/vmi/scripts/execWithPrivilege/smhapstatus.sh  &>/dev/null
HArc=$?
echo "HA get status return code is $HArc" >>$SAVE_LDAP_LOG

if [ $HArc -eq 4 ]; then
   echo "HA is not configured" >>$SAVE_LDAP_LOG
   # stop the ldap server
   echo "Stop local LDAP server:" >>$SAVE_LDAP_LOG
   /etc/init.d/ldap stop >>$SAVE_LDAP_LOG 2>&1
   # check pid file to see if ldap server ended, if not, kill the process
  if  [ -f $PID_FILE ]; then
   echo "'$PID_FILE' file exist" >>$SAVE_LDAP_LOG
   echo "ldap server still running, try to kill the process:" >>$SAVE_LDAP_LOG 
   killall slapd >>$SAVE_LDAP_LOG 2>&1
 fi
else
 if [ $HArc -eq 0 ]; then
  echo "HA node is active" >>$SAVE_LDAP_LOG
  echo "chrg -o offline" >>$SAVE_LDAP_LOG
  chrg -o offline $RGname >>$SAVE_LDAP_LOG 2>&1
  wait_for_rg_offline 
  RC=$?
  echo "Wait for ldap to go offline returned $RC" >>$SAVE_LDAP_LOG
  if [ $RC -eq 0 ]; then
       echo "ldap server is offline" >>$SAVE_LDAP_LOG
   else
     echo "ldap server failed to go offline" >>$SAVE_LDAP_LOG
  fi
 fi
fi
 
if [ $HArc -eq 3 ]; then
  echo "HA node is passive" >>$SAVE_LDAP_LOG
fi

# again check pid file to see if ldap server ended, if not, exit with error
# ldap server will not be running on the passive node
 
if  [ -f $PID_FILE ]; then
  echo "Cannot stop ldap server! Exit backup" >>$SAVE_LDAP_LOG
  echo "********Ldap server backup failed `date` ********" >>$SAVE_LDAP_LOG
  exit 1
fi


# Do this if HA is not configured or this is the active node
if [ $HArc -eq 4 -o $HArc -eq 0 ]; then
 # start to backup ldap server data to file /spare/ldap/backup.ldif
  echo "Run slapcat to back-up LDAP config to $SAVE_LDAP_BACKUP_DIR" >>$SAVE_LDAP_LOG
  /usr/sbin/slapcat -v -n 1 -l $SAVE_LDAP_BACKUP_DIR >>$SAVE_LDAP_LOG 2>&1
fi

echo "Start local LDAP server:" >>$SAVE_LDAP_LOG

if [ $HArc -eq  0 ]; then
  echo "chrg -o online" >>$SAVE_LDAP_LOG
  chrg -o online $RGname  >>$SAVE_LDAP_LOG 2>&1
  wait_for_rg_online   >>$SAVE_LDAP_LOG 2>&1
  RC=$?
  echo "Wait for ldap to go online returned $RC">>$SAVE_LDAP_LOG
  if [ $RC -eq 0 ]; then
     echo "ldap server is online" >>$SAVE_LDAP_LOG
   else
     echo "ldap server failed to go online" >>$SAVE_LDAP_LOG
  fi
fi

if [ $HArc -eq  4 ]; then
 #start ldap server
  /etc/init.d/ldap start >>$SAVE_LDAP_LOG 2>&1
fi

#check ldap server status
 echo "Local LDAP server status:" >>$SAVE_LDAP_LOG
if [ $HArc -eq  4 -o $HArc -eq  0 ]; then
 /etc/init.d/ldap status >>$SAVE_LDAP_LOG 2>&1
 echo "********Successfully backed up LDAP server `date` ********" >>$SAVE_LDAP_LOG
 if  [ ! -f $PID_FILE ]; then
  echo "********Failed to start LDAP server `date` ********" >>$SAVE_LDAP_LOG
 fi
fi
exit 0