#! /bin/bash
#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

export PATH=/usr/bin:/sbin:/usr/sbin:/bin:/usr/smrshbin/:$PATH

NETWORK_CHECK="/usr/smrshbin/lsnetcfg -n"
CONFIG_FILE_DIR="/etc/xinetd.d/"

port=$1
data_network_interface=$2
to_port=$3
to_host=$4
client_ip=$5
if [[ $# -ne 5 ]];then
	client_ip="na"
fi
rc=0

#These ensure that port forwarding is enabled on the FSM
#TODO See if we can get rid of these
echo '1' > /proc/sys/net/ipv4/conf/eth1/forwarding
echo '1' > /proc/sys/net/ipv4/conf/eth0/forwarding
echo '1' > /proc/sys/net/ipv4/ip_forward


# We'll open the port after configuring the forwarding rules, that way if the forwarding fails we never open the port
if [[ ($(iptables -L $data_network_interface-FIREWALL-SETTING | grep -nc $port) -eq 0 || "$client_ip" != "na") && $(echo $client_ip | grep -c ':') -eq 0 ]];then 
	if [[ "$client_ip" == "na" ]];then
		iptables -I $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	else
		iptables -I $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port -s $client_ip -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	fi
	rc=$(($rc+$?))
fi

if [[ $rc -eq 0 && ($(ip6tables -L $data_network_interface-FIREWALL-SETTING | grep -nc $port) -eq 0 || "$client_ip" != "na") && $(echo $client_ip | grep -c ':') -ge 1 ]];then 
	ip6tables -I $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port -s $client_ip -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	rc=$(($rc+$?))
fi

# If opening the port failed, then we should exit out and return an error status
if [[ $rc != 0 ]];then
	# If the port fails to open there is no point in having an extranious forward rule laying around so we'll close it
	if [[ "$client_ip" == "na" ]];then
		iptables -D $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	else
		iptables -D $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port -s $client_ip -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	fi
	ip6tables -D $data_network_interface-FIREWALL-SETTING -p tcp -i $data_network_interface --dport $port  -s $client_ip -j ACCEPT -m state --state NEW,ESTABLISHED,RELATED
	exit $rc
fi

if [[ $(ls -l $CONFIG_FILE_DIR | grep -c $port) -eq 0 ]]; then
	# Determine if we are dealing with an IPv4 or IPv6 host address for the destination
	if [[ $(echo "$to_host" | grep :) -eq 0 ]]; then
		flag="IPv4"
	else
		flag="IPv6"
	fi
	
	# It doesn't really matter that the below doesn't check source IPs, because that is regulated by the Firewall rules
	printf "# default: on\n
		service $port\n
		{\n
			disable		= no\n
			flags		= $flag\n
			type		= UNLISTED\n
			socket_type     = stream\n
	        	protocol        = tcp\n
	        	user            = nobody\n
	        	wait            = no\n
		        port            = $port\n
			redirect        = $to_host $to_port\n
		}" > $CONFIG_FILE_DIR$port
fi
	
# Get the process ID of xinetd 
xinetd_pid=$(ps axuww | grep -i xinetd | grep -v grep | sed 's/^[[:space:]]*//;s/[[:space:]]*$//;s/[[:space:]]\+/,/g' | cut -d',' -f 2)
if [[ $(ps axuww | grep -i xinetd | grep -v grep | wc -l) -eq 0 ]]; then
	#xinetd is not running, so start it
	/etc/init.d/xinetd start
else 
	#xinetd is already running, so send it the signal to refresh itself
	#This will force xinetd to read the new configuration files without killing the existing process, so existing port forwarding sessions continue to work
	kill -SIGHUP $xinetd_pid
fi


exit $rc
