#!/bin/bash

# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# Code added for HA handling (chrg command).

PID_FILE=/var/run/slapd/slapd.pid
source /opt/ibm/director/lwi/runtime/itme/eclipse/plugins/bin/RGFunctions.sh


#This script is called during an update of FSM. If HA is configured, we must use the chrg command
# instead of enable/disable the ldap watchdog and /etc/init.d/ldap. TLS is in charge of ldap
# when HA is configured. Logging will go to the update install log. See installFSMUpdate.sh for more info.

###############################################################################################
# Global script variables
# Log to /opt/ibm/director/lwi/logs/fsmUpdateLog_yymmdd.log
TODAY=`date '+%y%m%d'`
# Collected by pedbg.  Do not change this unless you change fsmCopyLogs script
# in com.ibm.itme.build/appliance/pedbg
#LOGFILE=/opt/ibm/director/lwi/logs/fsmUpdateLog_${TODAY}.log
LOGFILE=/home/pe/fsmUpdateLog_${TODAY}.log

DATETIME=`date`   
#

# Place stdout and stderr in LOGFILE and to console
exec >> $LOGFILE 2>&1

# Send each command in script to STDOUT
set -x

# This routine is called if a command has an error in the script.
handle_error() {
    
    echo "$(basename $0)  caught error on line : $1 command was: \"$2\" : Error was " $3
    #exit $3
}

# Turn on command error reporting
turn_on_error_chk() {
  trap 'handle_error $LINENO "$BASH_COMMAND" $?' ERR
}

# Turn off command error checking
turn_off_error_chk() {
  trap " " ERR
} 

# Parameters
# $1 = 1  start ldap
# $1 = 0  stop ldap
#
echo "installFSMUpdateLdap.sh : parm is $1"

## Stop ldap server
if [ $1 -eq 0 ] ; then
  	echo "installFSMUpdateLdap.sh : Stop local LDAP server" 
  	/usr/smrshbin/disableLdapWatchdog
	# stop the ldap server
	/etc/init.d/ldap stop  
	# grep for an active ldap server process to see if ldap server ended, if not, kill the process
	ps -ef | grep slapd | grep -v "grep" | grep "/usr/lib/openldap/slapd" > /dev/null
	LDAP_ACTIVE=$?
	if [ $LDAP_ACTIVE -eq 0 ]; then
   		echo "installFSMUpdateLdap.sh : ldap server still running, try to kill the process:" 
   		killall slapd
	fi
else 
 	echo "installFSMUpdateLdap.sh : Start local LDAP server:" 
	#start ldap server
	
	echo "Start local LDAP server:"  
	/etc/init.d/ldap start  
	/usr/smrshbin/enableLdapWatchdog
 	#check ldap server status
 	echo "installFSMUpdateLdap.sh : Local LDAP server status"  
 	/etc/init.d/ldap status  
fi


exit 0