#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2010 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

#!/bin/bash

# Return Code
#0: Start was successful
#1: Start failed with error
#2: LWI is already running; start failed

LWI_BIN_DIR=/opt/ibm/director/lwi/bin
LWI_START_OPTION="profile.lap=uimin,uistandard,rest,lap com.ibm.lwi.profile=profile.lap com.ibm.pvc.webcontainer.http.sessionCookieSecure=false"
PLATFORM_XML_FILE=/opt/ibm/director/lwi/conf/org.eclipse.update/platform.xml
PLATFORM_XML_BACKUP_FILE=/opt/ibm/director/lwi/conf/org.eclipse.update/.platform.xml.backup
CERTIFICATE_FLAG_FILE=/opt/ibm/director/vmi/data/.httpsCertificateCreated

# Update the web certificates to SHA256 encryption -removed XB099686 and replaced with creating a new certificate per system
if [ ! -f $CERTIFICATE_FLAG_FILE ]; then
	touch $CERTIFICATE_FLAG_FILE
	#echo "remove the current certificate"
	/opt/IBMJava/bin/keytool -delete -keystore /opt/ibm/director/data/security/keystore/ibmjsse2.jks -storepass ibmpassw0rd -alias lwiks 
	echo "add new certificate"
/opt/IBMJava/bin/keytool -genkey -alias lwiks -keyalg RSA -dname "CN=Flex System Manager HTTPS Server,O= Generated by IBM Firmware, L=Rochester, ST=MN, C=US" \
 -keystore  /opt/ibm/director/data/security/keystore/ibmjsse2.jks -validity 9131 -storepass ibmpassw0rd -keypass ibmpassw0rd -keysize 2048 -sigalg SHA256withRSA
fi
# End of web certificate copy


# Checking LWI status
STATUS=`${LWI_BIN_DIR}/lwistatus.sh`
if [ ${STATUS} == "ACTIVE" ] ; then
	echo "LWI must be stopped!"
	exit 2
fi
if [ ! -f $PLATFORM_XML_BACKUP_FILE ]; then
	cp $PLATFORM_XML_FILE $PLATFORM_XML_BACKUP_FILE
fi
# Starting LWI
RETVAL=0
echo  "Starting LAP LWI..."
nohup ${LWI_BIN_DIR}/lwistart.sh ${LWI_START_OPTION} >/dev/null 2>&1 &
LAP_LWI_PID=`ps -efww|grep "${LWI_START_OPTION}"|grep -v grep|awk {'print $2'}`
RETVAL=$?
if [ $RETVAL -eq 0 ]; then
	STATUS=`${LWI_BIN_DIR}/lwistatus.sh`
	COUNT=0
	while [ "$STATUS" != "ACTIVE" ]; do
		sleep 5
		COUNT=`expr $COUNT + 1`
		STATUS=`${LWI_BIN_DIR}/lwistatus.sh`
		if [ $COUNT -ge 80 ]; then
			echo "LWI is taking too long to start."
			echo "LAP LWI did not start successfully."
			exit 1
		fi
	done
	echo "LAP LWI is started."	
	/opt/ibm/director/vmi/bin/launchBrowser https://localhost:8422/ibm/console
	# kick off SLP script to gather all addresses for setupwizard duplicate address validation
	/opt/ibm/director/vmi/bin/retrieveIpAddressesFromSlp.sh &
	
	licensefile="/opt/ibm/director/vmi/data/.licenseaccepted"
	count=10
	# Wait some time before checking, because there is a chance that we will attempt to launch firefox twice
	sleep 10
	while [ ! -f $licensefile ]; do
		#if we detect user accidentally closed license local console browser, restart it
		startid=`ps auxww | grep "^cons" | grep "firefox" | awk '{print $2}'`
    	if [ -z "$startid" ]; then
			/opt/ibm/director/vmi/bin/launchBrowser https://localhost:8422/ibm/console
    	fi

		sleep 3
	done
else
	echo "Error occurred during starting LAP"
	exit 1
fi

exit 0
