#! /bin/bash
#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=manageFilesonHost
. /usr/bin/gettext.sh

export PATH=/usr/bin:/bin:$PATH

FOUNDATIONS_DIR="/media/foundations"
MGMT_NET_UTIL="/opt/ibm/director/vmi/scripts/execWithPrivilege/mgmtNetUtil"
FSM_BACKUP_LOG="/var/log/fsm_backuplog"

path="$(readlink -m /dev/disk/by-path/$(/opt/ibm/director/vmi/scripts/execWithPrivilege/mgmtNetUtil runcommand ls /dev/disk/by-path/ -lah | grep -iP 'sas.*part3' | sed -r 's/.*->(.*)/\1/g' | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'))"

validate_partition(){
	doesPartitionExist=$($MGMT_NET_UTIL runcommand fdisk -l 2>&1 | grep $path | wc -l)
    if [[ $doesPartitionExist -eq 0 ]]; then
    	printf "`gettext 'Backup partition does not exist on the drive. This must be corrected by running a recovery of the HDD. Backup attempt terminated'`\n" 
    	exit 2
    fi
    
    ismounted=`$MGMT_NET_UTIL runcommand mount | grep $path | wc -l`
    ismediaused=`$MGMT_NET_UTIL runcommand mount | grep /media | wc -l`
    mountedtogether=`$MGMT_NET_UTIL runcommand mount | grep $path | grep /media | wc -l`
    
    if [[ $ismounted -eq 0 ]];then
    	$MGMT_NET_UTIL runcommand $CHECK_PARTITION_FORMAT >> $FSM_BACKUP_LOG 2>&1
    	notFormatted=$?
    	if [[ $notFormatted -ne 0 ]]; then
    		printf "`date +%m/%d/%Y%t%H:%M:%S` - `gettext ' Format of backup partition required, beginning format process.'`\n"
    		$MGMT_NET_UTIL runcommand mkfs -t ext4 -L Backup $path >> $FSM_BACKUP_LOG 2>&1
    		printf "`date +%m/%d/%Y%t%H:%M:%S` - `gettext ' Format of backup partition complete.'`\n"
    	fi
    fi
    
    if [[ $ismounted -eq 0 && $ismediaused -eq 0 ]]; then
    	$MGMT_NET_UTIL runcommand mount $path /media >> $FSM_BACKUP_LOG 2>&1
    elif [[ $mountedtogether -eq 0 ]]; then
    	$MGMT_NET_UTIL runcommand umount /media >> $FSM_BACKUP_LOG 2>&1
    	$MGMT_NET_UTIL runcommand mount $path /media >> $FSM_BACKUP_LOG 2>&1
    fi
    
    if [[ $($MGMT_NET_UTIL runcommand mount | grep $path | grep /media | wc -l) -eq 0 ]]; then
    	printf "`gettext 'Failed to mount the backup partition. Terminating backup attempt.'`\n"
    	exit 3
    fi
}

if [[ $# -lt 1 || $# -gt 3 ]];then
	printf "Options: -d (self-destruct this script),\n -p <file with full path> (move the given file to the host),\n -l (list the files being stored on the host),\n -g <file name> <path> (move the given file from the host to the specified path on the vm)\n"
	exit 1
fi

# Read given file (hidden command to be used by FSM code only)
if [[ "$1" == "-r" ]];then
	file_path=$FOUNDATIONS_DIR/$2
	
	validate_partition

	$MGMT_NET_UTIL runcommand cat $file_path | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'
	exit $?
fi

# Self-Destruct
if [[ $1 == "-d" ]];then
	rm /usr/smrshbin/manageFilesonHost
	exit 0
fi

# Put file to host
if [[ $1 == "-p" ]];then
	file_path=$2
	file_name=$(echo $file_path | sed 's/\/[a-zA-Z0-9\/]*\///g')

	validate_partition

	$MGMT_NET_UTIL runcommand mkdir $FOUNDATIONS_DIR
	$MGMT_NET_UTIL copyfiletohost $file_path $FOUNDATIONS_DIR/$file_name

	exit $?
fi

# Get file from host
if [[ $1 == "-g" ]];then
	file_name=$2
	file_path=$3

	[[ $file_path != */ ]] && file_path="$file_path/"

	validate_partition

	$MGMT_NET_UTIL copyfilefromhost $FOUNDATIONS_DIR/$file_name $file_path$file_name

	exit $?
fi

# List files on host
if [[ $1 == "-l" ]];then
	validate_partition

	$MGMT_NET_UTIL runcommand ls -lAh $FOUNDATIONS_DIR
	exit $?
fi

printf "Options: -d (self-destruct this script),\n -p <file with full path> (move the give file to the host),\n -l (list the files being stored on the host),\n -g <file name> <path> (move the given file from the host to the specified path on the vm)\n"
exit 1
