#!/bin/bash
#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

SCRIPT_PATH="/opt/ibm/director/proddata/itme/datamgmt/"
DEAMON_COMMAND="pmonitord"
ADD_COMMAND="addPort"
NETWORK_CHECK="/usr/smrshbin/lsnetcfg -n"
IP_INFO_FILE="/opt/ibm/director/lwi/conf/overrides/.ipAddresses.prop"

ip_to_check=$1
deamon_running=$(ps axuww | grep -c $DEAMON_COMMAND)

if [[ $deamon_running -lt 2 ]];then
	#****************************Start Config File Setup****************************
	#This section is designed to setup the config file required by the port monitor deamon
	#The file is expected to contain the IP addresses on the FSM's Data Network interface
	#as well as the number of IP addresses

	#Obtain the list of IP addresses as an array
	localIPAddresses=$($NETWORK_CHECK | grep -P 'ipv4addr_eth1|ipv6addr_eth1' | sed 's/ipv4addr_eth1://;s/ipv6addr_eth1://;s/^eth1[[:space:]]*//;s/,/\n/g;s/[[:space:]]*$//;s/^[[:space:]]*//;s/[[:space:]]*$//;s/\/[[:digit:]][[:digit:]]//g')
	output_to_file=""
	numberAddresses=0
	for address in $localIPAddresses
	do
		# Convert the IPv6 into it's long form (e.g. 2002::1 becomes 2002:0:0:0:0:0:0:1)
		# that is required by the port monitor code. This was decided because doing 
		# string manipulations is easier in shell scripting, then in C
		short_ipv6=$(echo $address | grep -c ::)
		if [[ $short_ipv6 -gt 0 ]];then
			# This code determines how many 0's are hidden inside the ::
			# It determines that value in the following way:
			# 1) determine the number of ':' in the address
			# 2) we know that a fully expanded IPv6 address will have 7
			# ':'s so the number of ':'s missing will be 7 minus the 
			# number that we currently have. From there we know that the
			# number of 0's to be added is 1 plus the number of ':'s to 
			# be added.
			number_delimiters=$(echo $address | grep -o : | wc -l)
			number_zeros_needed=$(($((7-$number_delimiters))+1))
			expansion_string=":"
			counter=0
			while [[ $counter -lt $number_zeros_needed ]]
			do
				expansion_string=$expansion_string"0:"
				counter=$(($counter+1))
			done
			address=$(echo $address | sed "s/::/${expansion_string}/g")
		fi
		output_to_file=$output_to_file$address"\n"
		numberAddresses=$(($numberAddresses+1))
	done
	output_to_file=$numberAddresses"\n"$output_to_file
	printf $output_to_file > $IP_INFO_FILE
	#****************************End Config File Setup****************************

	$SCRIPT_PATH$DEAMON_COMMAND >> /var/log/portforward_log 2>&1 &
fi

counter=0
while [[ $(ls -l /tmp | grep -c ".pm_fifo") -eq 0 && counter < 5 ]];do
	sleep 1
	counter++
done
returnvalue=$($SCRIPT_PATH$ADD_COMMAND $ip_to_check)
echo $returnvalue
exit $returnvalue
