#!/bin/sh
   # This script will summarize the ps* files from a waitDataCollector.sh run. 
   # and find the javacores for the threads using the most cpu.
   #
   # Input: assume current directory contains tar.gz file from waitDataCollector.sh
   # Output: psSummedThreads.out - file of uniqued threadIDs with total cpu over all collected intervals
   #         anzWaitData.out - Entries of threadID and total cpu used followed by all the stacks found for the 
   #                           threadID entry
   
   
   anzProcessData() {
   # This function will summarize the ps* files from a waitDataCollector.sh run. 
   # and find the javacores for the threads using the most cpu.
   #
   # Input: Directory that contains the ps* and javacore files.
   # Output: psSummedThreads.out - file of uniqued threadIDs with total cpu over all collected intervals
   #         anzWaitData.out - Entries of threadID and total cpu used followed by all the stacks found for the 
   #                           threadID entry
   
   # First parm is the directory were the ps* and javacore* files can be found.
   inputDir=$1
   #define all the output files
   psFile=$inputDir/psAllThreadsOutput.txt
   psSummedOut=$inputDir/psSummedThreads.out
   anzWaitOut=$inputDir/anzWaitData.out
   
   #Merge all the psThreadsOut files.
   echo "Merging $inputDir/psThreadsOutput* files"
   cat $inputDir/psThreadsOutput* > $psFile   
   #sort by the first field, thread id (k1)
   echo "Sorting $psFile by threadID"
   sort -k1 $psFile -o $psFile
   echo "Summing cpu percent in $psFile by threadID"
   echo "Thread Cpu% Priority" > $psSummedOut;
   firstThread=1;
   while read threadID skip1 skip2 cpuPercent priority theRest;
   do
   #If a valid threadID (numeric)
    if [[ $threadID =~ ^[0-9]+$ ]]; then
      if [[ firstThread = 1 ]]; then
      threadIDlast=$threadID;
      cpuTotal=$cpuPercent;
      firstThread=0;
      #else if the current thread is equal to the last thread, then sum the cpu.
     elif [[ $threadID = $threadIDlast ]]; then
       cpuTotal=`echo "$cpuTotal + $cpuPercent" | bc `;
     else
       # if a new thread, output the last threads info and reset the summation variables.
       echo $threadIDlast $cpuTotal $priority >> $psSummedOut;
       threadIDlast=$threadID;
       cpuTotal=$cpuPercent;
     fi # if threads are the same
   fi # valid thread
   done <$psFile;
   # write out the last record 
   echo $threadLast $cpuTotal >> $psSummedOut;
   
   echo "Sorting $psSummedOut by cpuPercent"
   #sort the summed file of threads by the 2nd field (cpuPercent=k2)
   sort -k 2 -n -r $psSummedOut -o $psSummedOut; 
   
   #clear the output file
   echo "Analysis of $inputDir" > $anzWaitOut;
   echo " " >> $anzWaitOut
   #cpu percent needs to be at least 5.0 to be of any interest.
   cpuFloor=5.0;   
   #read the summed file of threads
   echo "Finding javacores for all threads using more than $cpuFloor percent cpu"
   while read threadID cpuPercent priority;
   do
    # if threadID is a real number (and not a header or other garbage)
    if [[ $threadID =~ ^[0-9]+$ ]]; then
    if [[ $threadID > 0 ]]; then
     cpuCompare=`echo "$cpuPercent > $cpuFloor" | bc`
     # if cpuPercent is > cpuFloor
     if [ $cpuCompare -eq 1 ] ; then    
       #convert decimal threadID to hex
       hexThread=`echo "obase=16; $threadID"|bc`
       echo "Thread: $threadID   Total CPU used: $cpuPercent" >> $anzWaitOut;
       #find the hex threadID in the java core and write out the two lines before and 20 lines after 
       grep -h -B2 -A20 "native thread ID:0x$hexThread"  $inputDir/javacore*  >> $anzWaitOut;
       echo " " >>  $anzWaitOut;
       echo " " >>  $anzWaitOut;
       echo " " >>  $anzWaitOut;
     fi  # if cpuCompare -eq 1
     fi # if threadID > 0
    fi # if numeric
   done <$psSummedOut;
   }
   # end of anzProcessData()
   
   anzDate=`date '+%m-%d-%Y_%H:%M' 2>/dev/null`
   tarGzFile=waitData.tar.gz
   tarGzDir="anzWait_"$anzDate
   mkdir $tarGzDir
   tar -xf $tarGzFile -C $tarGzDir
   
   PIDfile=$tarGzDir/PIDs.out;
   
   ls $tarGzDir | grep PID_ > $PIDfile
   
   #read PIDfile   
   while read tmpDir; do
     PIDdir=$tarGzDir/$tmpDir
     anzProcessData $PIDdir;
     cp $PIDdir/anzWaitData.out $PIDdir.anzWaitData.out
   done <$PIDfile
   echo "Output is in $tarGzDir/PIDxxxxx.anzWaitData.out files"
   mv waitData.tar.gz "waitData."$anzDate".tar.gz"
   echo "waitData.tar.gz moved to waitData."$anzDate".tar.gz"
