#!/bin/bash
#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=scheduleBackup
export PATH=/usr/bin:/sbin:/usr/sbin:/bin:/usr/smrshbin/:$PATH
. /usr/bin/gettext.sh

FSM_BACKUP_LOG="/var/log/fsm_backuplog"
#TODO Need to add an options for SFTP
print_usage() {
	if [[ $1 == 'h' ]];then
		printf "`gettext 'Usage:\n\t '`"
		printf "`gettext 'scheduleBackup -h \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s sdate -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s now -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s sdate -f WEEKLY\n\t -d {sun|mon|tue|wed|thu|fri|sat} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -h \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s sdate -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s now -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s sdate -f WEEKLY\n\t -d {sun|mon|tue|wed|thu|fri|sat} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -D \n\t scheduleBackup -l'`\n\n\n"
		printf "`gettext 'IMPORTANT:\n\t Using this command to generate a new backup schedule will overwrite existing backup schedules. \n\n\nOptions: \n\t -h, --help\n\t\t Print this help\n\n\t -t, --type {USB|LOCAL|SFTP}\n\t\t Specify what media to use for the scheduled backups of the IBM Flex System Manager. USB: store the backup images onto a USB device connected to the IBM Flex System Manager Node. LOCAL: Backup to local storage on the IBM Flex System Manager node. SFTP: Backup is store on the external SFTP server.\n\n\t -s, --start-date sdate\n\t\t Give the date and time to start the first scheduled backup. sdate Either a date of the format mm/dd/yyyy/hh/MM (m = month {1-12}, d = day of month {1-31}, y = year, h = hour {0-23}, M = minute {0-59}) or it can be the keyword now (if now is given it will set the first backup to run right after this command has finished.\n\n\t -f, --frequency {DAILY|WEEKLY|MONTHLY|YEARLY}\n\t\t Determines how often the backups are supposed to run, once a day (DAILY), once a week (WEEKLY), once a month (MONTHLY), or once a year (YEARLY). If WEEKLY is specified then the -d option is required.\n\n\t -d, --day {sun|mon|tue|wed|thu|fri|sat}\n\t\t Specify the day of the week that the scheduled backup is supposed to run, Sunday (sun), Monday (mon), Tuesday (tue), Wednesday (wed), Thursday (thu), Friday (fri), Saturday (sat). This option is required with using -f WEEKLY, it is ignored in all other cases.\n\n\t -e, --end-date edate\n\t\t The last day to run the scheduled backup. edate a date in the form of mm/dd/yyyy (m = month {1-12}, d = day of month {1-31}, y = year).\n\n\t -w, --weeks weeks\n\t\t The number of weeks that the scheduled backups are supposed to be run.\n\n\t -m, --months months\n\t\t The number of months that the scheduled backups are supposed to be run.\n\n\t -y, --years years\n\t\t The number of years that the scheduled backups are supposed to be run.\n\n\t -D, --Delete-schedule\n\t\t Delete the current backup schedule\n\n\t -l, --list\n\t\t List the current scheduled backups. Will either state that there are no backups or it will return the date and time of the next scheduled backup.\n\n\t -P,--Path path\n\t\t Specify the SFTP server and the path on the SFTP server for SFTP backups. path should be of the form server_ip/path_on_server_to_backups\n\n\t -u, --userid user\n\t\t The user ID of the user to access the SFTP server for SFTP backups.\n\n\t -p, --password password\n\t\t The password for user to access the SFTP server. If this parameter is non used you will be prompted to enter a password.'`\n"
	else
		printf "`gettext 'Usage:\n\t '`"
		printf "`gettext 'scheduleBackup -h \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s sdate -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s now -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t {USB|LOCAL|SFTP} -s sdate -f WEEKLY\n\t -d {sun|mon|tue|wed|thu|fri|sat} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -h \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s sdate -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s now -f {DAILY|MONTHLY|YEARLY} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -t SFTP -P server_path -u user [-p password] -s sdate -f WEEKLY\n\t -d {sun|mon|tue|wed|thu|fri|sat} [-e edate | -w weeks | -m months | -y years] \n\t scheduleBackup -D \n\t scheduleBackup -l'`\n\n\n"
	fi

	/opt/ibm/director/vmi/bin/execWithPrivilege auditLogger scheduleBackup 2
        exit 0
}

# Format date to YYYYMMDDHHMM so we can use it to compare dates
format_date() {
	local paramTime=$1

	local -a darray
  	local temp
  	local out

  	# Split the variable into an array with / as our separator
  	IFS=/ read -a darray <<< "$paramTime"

  	# swapping years and days
  	temp=${darray[0]}
  	darray[0]=${darray[2]}
  	darray[2]=${darray[1]}
  	darray[1]=$temp

  	# loop through and build the output string
  	# use printf command to make sure the string is at least 2 wide.
  	out=""
  	for element in "${darray[@]}"; do
    		out="$out"$(printf "%02d" "${element#0}")
  	done

  	echo $out		
}

#The path to the DM properties file which holds all the other path information
DM_PROPERTIES_FILE="/opt/ibm/director/lwi/conf/overrides/.datamgmt.properties"

#The name of the property in the DM properties file, that we are interested in
PROPERTY_PROPFILE="backups_propfile_path"
PROPERTY_SCRIPTPATH="backups_script_path"


#The file name for the backup properties file
PROPERTY_FILE_NAME=".backupSchedule.properties"

backupPropertiesFile=$(grep $PROPERTY_PROPFILE $DM_PROPERTIES_FILE 2>&1 | sed "s/$PROPERTY_PROPFILE=//;s/\r//")$PROPERTY_FILE_NAME
backupScriptsPath=$(grep $PROPERTY_SCRIPTPATH $DM_PROPERTIES_FILE 2>&1 | sed "s/$PROPERTY_SCRIPTPATH=//;s/\r//")

# Backup properties file, Porperty IDs
PROPID_SAFEMODE="safeMode"
PROPID_FIRSTBACKUP="firstDate"
PROPID_ENDDATE="endDate"
PROPID_DAILY="daily"
PROPID_USBPATH="usbPath"
PROPID_CRONSTRING="cronString"
PROPID_FOR="for"
PROPID_UNITS="unit"
PROPID_SFTP="sftp"
PROPID_PASSWORD="password"

number_weeks=-1
number_months=-1
number_years=-1
frequency="na"
start_date="na"
end_date="na"
day_of_week="na"
userid="na"
password="na"
server_path="na"
#The cron command to run of the form "MM hh dd mm <comma separated list of days in week> <command>"
cron_string=""

get_increment() {
	increment=""
	starting_time=$1 
	formatted_startTime=$(format_date $starting_time)
	daily=$(grep $PROPID_DAILY $backupPropertiesFile | sed "s/$PROPID_DAILY=//")
	if [[ "$daily" != "true" ]];then
		cronstring=$(grep $PROPID_CRONSTRING $backupPropertiesFile | sed "s/$PROPID_CRONSTRING=//")
		months=$(echo "$cronstring" | cut -d' ' -f 4)
		days=$(echo "$cronstring" | cut -d' ' -f 5)
		# if both the day_of_week, month field in the cron string are * then we know it is a monthly backup
		# if months is not * but day_of_week is, then we know it's a yearly backup, but if days is not *
		# but months is, then it's a weekly backup
		if [[ "$days" == "*" && "$months" == "*" ]];then
			# If it's a up coming backup time then don't increment
			if [[ $formatted_currTime -gt $formatted_startTime ]]; then
				increment="+1 months"
			fi
		elif [[ "$days" == "*" ]];then
			# If it's a up coming backup time then don't increment
			if [[ $formatted_currTime -gt $formatted_startTime ]]; then
				increment="+1 years"
			fi
		else
			local currTime=$(echo $current_time | cut -d'/' -f -3)
			# Set nextBackup to current time instead of starting time 
			# because we need to calculate the very next backup as of current time
			nextBackup=$(echo $current_time | cut -d'/' -f -3)"/"$(echo $start_time | cut -d'/' -f 4-)
			# get what day of the week the backup starts
			starting_day_week=$(date --date="$currTime" +%w)

			# Use the below formula to determine how many days to add to have the date function start on the right day of the week
			: $((difference=$days-$starting_day_week))
			: $((difference=$difference+7))
			: $((difference=$difference%7))
			increment="+$difference days"
		fi
	else
		# backup started in the past
		if [[ $formatted_currTime -gt $formatted_startTime ]]; then
			# need to compare HHMM and determine next backup as today or tmw 
			starting_hrs=$(echo $starting_time | cut -d'/' -f 4- | sed 's#/##g') 	 	
			curr_hrs=$(echo $current_time | cut -d'/' -f 4- | sed 's#/##g') 	 	
			# Starting backup has passed, set nextBackup to today then increment one day 
			nextBackup=$(echo $current_time | cut -d'/' -f -3)"/"$(echo $start_time | cut -d'/' -f 4-)			
			if [[ $curr_hrs -gt $starting_hrs ]]; then 	 	  	 	
				increment="+1 days" 	 	
			fi 	
		fi 
	fi
}

ARGS=`getopt -ua -n $COMMAND_NAME -o "h:t:s:e:w:m:y:f:d:D:l:u:p:P" -- $*`
if [[ $? -ne 0 ]];then
	/opt/ibm/director/vmi/bin/execWithPrivilege auditLogger scheduleBackup 1
	exit 1
fi
set $ARGS --

# Parse the arguments
while [ "$1" != "--" ];
do
	case "$1" in
		-h|--help) print_usage 'h';;
		-t|--type) # Type of backup USB | LOCAL | SFTP (not currently valid)
			shift
			type=$1
			if [[ "$type" != "USB" && "$type" != "LOCAL" && "$type" != "SFTP" ]];then
				print_usage
			fi
			;;
		-s|--start-date) # Start date. Should either be the word "now" or a date of the format mm/dd/yyyy/hh/MM (can also be mm/dd/yyyy in which case the current hour and minute will be used) m=month, d=day, y=year, h=hour, M=minute 
			shift
			start_date=$1
			echo "`date +%m/%d/%Y%t%H:%M:%S` start_date value: $start_date" >> $FSM_BACKUP_LOG 2>&1
			if [[ "$start_date" == "now" ]]; then
				start_date=$(date -d "+1 minute" +%m/%d/%Y/%H/%M)
			fi
			if [[ $start_date == [[:digit:]][[:digit:]]/[[:digit:]][[:digit:]]/[[:digit:]][[:digit:]][[:digit:]][[:digit:]] ]];then
				start_date=$start_date"/"$(date +%H/%M)
			elif [[ $start_date != [[:digit:]][[:digit:]]/[[:digit:]][[:digit:]]/[[:digit:]][[:digit:]][[:digit:]][[:digit:]]/[[:digit:]][[:digit:]]/[[:digit:]][[:digit:]] ]]; then
				print_usage
			fi
			;;
		-e|--end-date) # end date. Needs to be of the format mm/dd/yyyy m=month, d=day, y=year
			shift
			end_date=$1
			if [[ $end_date != [[:digit:]][[:digit:]]/[[:digit:]][[:digit:]]/[[:digit:]][[:digit:]][[:digit:]][[:digit:]] ]];then
				print_usage
			fi
			;;
		-w|--weeks) # Number of weeks to run
			shift
			number_weeks=$1
			;;
		-m|--months) # Number of months to run
			shift
			number_months=$1
			;;
		-y|--years) # Number of years to run
			shift
			number_years=$1
			;;
		-f|--frequency) # DAILY | WEEKLY | MONTHLY | YEARLY
			shift
			frequency=$1
			if [[ "$frequency" != "DAILY" && "$frequency" != "WEEKLY" && "$frequency" != "MONTHLY" && "$frequency" != "YEARLY" ]];then
				print_usage
			fi
			;;
		-d|--day) # Day of week to run when using -f WEEKLY. Of the format sun, mon, tue, wed, thu, fri, sat
			shift
			day_of_week=$1
			;;
		-D|--Delete-schedule) # Remove current schedules if one exists
			$backupScriptsPath"implementCron.sh" "-r" "launchBackup.sh"
			$backupScriptsPath"implementCron.sh" "-r" "backupHDD.sh"
			rm -f $backupPropertiesFile
			rc=$?
			/opt/ibm/director/vmi/bin/execWithPrivilege auditLogger scheduleBackup $rc
			exit $rc
			;;
		-l|--list) # Show when the next backup will be, or a message saying there are no scheduled backups
			if [[ -e $backupPropertiesFile ]];then
				current_time=$(date +%m/%d/%Y/%H/%M)
				formatted_currTime=$(format_date $current_time)
				start_time=$(grep -i $PROPID_FIRSTBACKUP $backupPropertiesFile | sed "s/$PROPID_FIRSTBACKUP=//")

				has_enddate=$(grep -ic $PROPID_ENDDATE $backupPropertiesFile)
				if [[ $has_enddate -ne 0 ]];then
					end_date=$(grep -i $PROPID_ENDDATE $backupPropertiesFile | sed "s/$PROPID_ENDDATE=//")
					# Add HH and MM from start time to end_time date
					end_time=$end_date"/"$(echo $start_time | cut -d'/' -f 4)"/"$(echo $start_time | cut -d'/' -f 5)
					formatted_endTime=$(format_date $end_time)
				fi							

				if [[ $has_enddate -ne 0 ]];then
					[[ $formatted_currTime -ge $formatted_endTime ]] && before_end=0 || before_end=1
				else 
						before_end=1
				fi
				# Determine if the current time is before the end date, if not there are no more backups, if yes then we need to compute when the next backup will be
				if [[ $before_end -eq 1 ]];then
					nextBackup=$start_time
					get_increment $start_time
					formatted_nextBackup=$(format_date $nextBackup)
					# This determines when the date and time of the next backup
					if [[ $has_enddate -ne 0 ]];then
						[[ $formatted_nextBackup -ge $formatted_endTime ]] && before_end=0 || before_end=1
					else 
						before_end=1
					fi
					if [[ $before_end -eq 1 ]];then
						next_backup_date_format=$(echo $nextBackup | cut -d'/' -f -3)" "$(echo $nextBackup | cut -d'/' -f 4)":"$(echo $nextBackup | cut -d'/' -f 5)
						nextBackup=$(date --date="$increment $next_backup_date_format" +%m/%d/%Y/%H/%M)
						formatted_nextBackup=$(format_date $nextBackup)
					fi

					if [[ $has_enddate -ne 0 ]];then
						[[ $formatted_nextBackup -ge $formatted_endTime ]] && before_end=0 || before_end=1
					else 
						before_end=1
					fi
					
					if [[ $before_end -eq 1 ]];then
						next_backup_date_format=$(echo $nextBackup | cut -d'/' -f -3)" "$(echo $nextBackup | cut -d'/' -f 4)":"$(echo $nextBackup | cut -d'/' -f 5)
						date_string=$(date --date="$next_backup_date_format"  +%c)
						printf "`gettext 'Next Backup:\n %s'`\n" "$date_string"
					else
						printf "`gettext 'No backups scheduled'`\n"
					fi
				else
					printf "`gettext 'No backups scheduled'`\n"
				fi
			else 
				printf "`gettext 'No backups scheduled'`\n"
			fi
			/opt/ibm/director/vmi/bin/execWithPrivilege auditLogger scheduleBackup 0
			exit 0
			;;
		-u|--userid)
			shift
			userid=$1
			;;
		-P|--Path)
			shift
			server_path=$1
			;;
		-p|--password)
			shift
			password=$1
			;;
		*) print_usage;;
	esac
	shift
done

# Here is a long list of different comparison tests to determine if there was a usage error
if [[ "$frequency" == "na" || ("$frequency" == "WEEKLY" && "$day_of_week" == "na") ]];then
	print_usage
fi

if [[ "$start_date" == "na" ]];then
	print_usage
fi

if [[ "$type" == "SFTP" && ("$userid" == "na" || "$server_path" == "na" || "$password" == "na") ]];then
	if [[ "$password" == "na" && "$userid" != "na" && "$server_path" != "na" ]];then
		# If the user doesn't enter a password as a parameter (which they probably don't want to for security reasons) they can enter it as a prompt.
		printf "`gettext 'Enter Password for SFTP server:'`\n"
		read -s -p "" password
	else
		print_usage
	fi
fi

if [[ "$type" == "SFTP" ]];then
	#Here we validate that we can indeed get access to the SFTP server
	server_host=$(echo $server_path | cut -d'/' -f 1)
	server_filepath=$(echo $server_path | cut -d'/' -f 2-)
	$backupScriptsPath"checkSftpCred.sh" $server_host "/"$server_filepath $userid $password >> /dev/null
	rc=$?
	if [[ $rc -eq 9 ]];then
		printf "`gettext 'The given server was not reachable. Please enter a reachable SFTP server.'`\n"
		exit 9
	elif [[ $rc -eq 10 ]];then
		printf "`gettext 'Either the given user ID or the given password does not work on the given SFTP server. Please enter a valid user ID and password'`\n"
		exit 10
	elif [[ $rc -eq 11 ]];then
		printf "`gettext 'The given directory does not exist on the SFTP server. Please enter a valid path that the given user ID has access to.'`\n"
		exit 11
	elif [[ $rc -eq 13 ]];then
		printf "`gettext 'An internal error occured. Contact support.'`\n"
		exit 13
	fi
fi

#Starting here we actually get down to business, instead of just doing parameter checking

is_indefinite=0
for_propvalue=-1
units_propvalue="na"
#This converts the number of weeks/months/years into an end date
if [[ "$end_date" == "na" ]];then
	dateParm=""
	if [[ $number_weeks -ne -1 ]];then
		dateParm="+$number_weeks weeks"
		for_propvalue=$number_weeks
		units_propvalue="week"
	elif [[ $number_months -ne -1 ]];then
		dateParm="+$number_months months"
		for_propvalue=$number_months
		units_propvalue="month"
	elif [[ $number_years -ne -1 ]];then
		dateParm="+$number_years years"
		for_propvalue=$number_years
		units_propvalue="year"
	else 
		is_indefinite=1
	fi
	# We want the user to be able to specify an indefinite scheduled backup (i.e. the scheduled backup never expires)
	if [[ $is_indefinite -ne 1 ]];then
		# The below tells the date command to use the date given by start_date, after removing the hour and minute fields because the date command doesn't like our formatting otherwise
		dateParm=$dateParm" "$(echo $start_date | sed 's/\/[[:digit:]][[:digit:]]\/[[:digit:]][[:digit:]]$//')
		end_date=$(date -d "$dateParm" +%m/%d/%Y)
	fi
fi

backup_command=""
remainder_cron=" "
if [[ "$type" == "USB" ]];then
	launchBackup="launchBackup.sh -u -s -1 -n"
	backup_command="$backupScriptsPath$launchBackup"
elif [[ "$type" == "LOCAL" ]];then
	launchBackup="backupHDD.sh -n"
	backup_command="$backupScriptsPath$launchBackup"
elif [[ "$type" == "SFTP" ]];then
	launchBackup="launchBackup.sh -f -s -1 -n -P $server_path"
	backup_command="$backupScriptsPath$launchBackup -i $userid"
fi

hour=$(echo $start_date | cut -d'/' -f 4)
minute=$(echo $start_date | cut -d'/' -f 5)
month=
day_of_month=
day_of_week_cron=

if [[ "$frequency" == "DAILY" ]];then
	day_of_month='*'
	month='*'
	day_of_week_cron="0,1,2,3,4,5,6"
elif [[ "$frequency" == "WEEKLY" ]];then
	if [[ "$day_of_week" == "na" ]];then
		print_usages
	fi
	day_of_month='*'
	month='*'
	day_of_week_cron="$(echo $day_of_week | sed 's/sun/0/;s/mon/1/;s/tue/2/;s/wed/3/;s/thu/4/;s/fri/5/;s/sat/6/')"
elif [[ "$frequency" == "MONTHLY" ]];then
	day_of_month="$(echo $start_date | cut -d'/' -f 2)"
	month='*'
	day_of_week_cron='*'	
elif [[ "$frequency" == "YEARLY" ]];then
	day_of_month="$(echo $start_date | cut -d'/' -f 2)"
	month="$(echo $start_date | cut -d'/' -f 1)"
	day_of_week_cron='*'
fi

cron_string="$minute $hour $day_of_month $month $day_of_week_cron $backup_command"
echo "cron_string value: $cron_string" >> $FSM_BACKUP_LOG 2>&1
# Write all the properties out to the properties file
echo "$PROPID_SAFEMODE=false" > $backupPropertiesFile
echo "$PROPID_FIRSTBACKUP=$start_date" >> $backupPropertiesFile
if [[ $is_indefinite -ne 1 ]];then
	echo "$PROPID_ENDDATE=$end_date" >> $backupPropertiesFile
fi
if [[ "$frequency" == "DAILY" ]];then
	echo "$PROPID_DAILY=true" >> $backupPropertiesFile
else
	echo "$PROPID_DAILY=false" >> $backupPropertiesFile
fi
if [[ "$type" == "USB" ]]; then
	echo "$PROPID_USBPATH=true" >> $backupPropertiesFile
else
	echo "$PROPID_USBPATH=false" >> $backupPropertiesFile
fi
if [[ $for_propvalue -ne -1 && "$units_propvalue" != "na" ]];then
	echo "$PROPID_FOR=$for_propvalue" >> $backupPropertiesFile
	echo "$PROPID_UNITS=$units_propvalue" >> $backupPropertiesFile
fi
if [[ "$type" == "SFTP" ]];then
	echo "$PROPID_SFTP=true" >> $backupPropertiesFile
	echo "$PROPID_PASSWORD=$password" >> $backupPropertiesFile
else 
	echo "$PROPID_SFTP=false" >> $backupPropertiesFile
fi
echo "$PROPID_CRONSTRING=$cron_string" >> $backupPropertiesFile 2>&1

/opt/ibm/director/agent/bin/lwiencoder.sh -filename $backupPropertiesFile -keylist $PROPID_PASSWORD >> /dev/null 2>&1

# Ensure that any existing scheduled backups are cleaned up, before starting the new job
$backupScriptsPath"implementCron.sh" "-r" "launchBackup.sh"
$backupScriptsPath"implementCron.sh" "-r" "backupHDD"
$backupScriptsPath"implementCron.sh" "-a" "$cron_string"

/opt/ibm/director/vmi/bin/execWithPrivilege auditLogger scheduleBackup 0
exit 0
