#!/bin/bash
# Licensed Materials - Property of IBM
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# This script is used by High Avilablity for 
# invocation as part of HA configuration or termination.
# When HA is configured, the script comments out entries related
# to ldap processes to prevent them from automatically
# being spawned off. 
# When HA is terminated, the inittab entries for ldap processes 
# are uncommented, allowing these processes to be respawned automatically 
# This is needed to make sure that we are unable to unmount /mirror0 and
# /dump directories when a node goes passive without having to kill ldap 
# processes that hold locks on files in these mounts.
# This script should be put in /opt/ibm/director/ha/plugins/conf/conf_ldap.sh

######conf_ldap.sh

# Enable logging
SMHA_DEBUG="ON"
PID=
RC=0
ldapphrase=ldap:

#include common exploiter code
EXPLOITER_COMMON_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh

if [ -f $EXPLOITER_COMMON_CODE ]; then
    source $EXPLOITER_COMMON_CODE
else
    echo "$PROGNAME:Exploiter common code not found."
    exit 1
fi


INITTAB=/etc/inittab
NEW_INITTAB=${INITTAB}.new.$$    # use $$ to create unique file name
BK_INITTAB=${INITTAB}.bk

cp $INITTAB $NEW_INITTAB                   # make copy
cp $INITTAB ${INITTAB}.bk                  # make backup

modify_service() {
	status=${1}
	
   case ${status} in 
      on) 
	      # add ldap to /etc/inittab
	     for f in $NEW_INITTAB
         do
          if grep $ldapphrase $NEW_INITTAB > /dev/null ;then
           echo "file \"$NEW_INITTAB\" contains $ldapphrase"
           sed -i 's?^#ldap.*$?ldap:2345:respawn:/opt/ibm/director/vmi/bin/createLdapWatchdog.sh 1>/dev/null 2>\&1?' $NEW_INITTAB
           else
            echo "file \"$NEW_INITTAB\" do not contains $ldapphrase"
            echo "ldap:2345:respawn:/opt/ibm/director/vmi/bin/createLdapWatchdog.sh 1>/dev/null 2>&1" | cat >> $NEW_INITTAB
          fi          
          smha_debug "added ldap to $NEW_INITTAB"
         done
         
         # add service to OS control and start it
            chkconfig_out=`/sbin/chkconfig --add ldap`
            rc=$?
            smha_debug "/sbin/chkconfig --add ldap: output: ${chkconfig_out} : rc: $rc"
            #start_out=`/etc/init.d/ldap start`
            #rc=$?
            #smha_debug "/etc/init.d/ldap start: output: ${start_out} : rc: $rc"
	      ;;
     off)
            # refresh the init task so it no longer respawn the server
            perl -pi -e "s/^ldap:2345/#ldap:2345/g" $NEW_INITTAB            
            
            smha_debug "remove ldap from $NEW_INITTAB"

            smha_debug "kill createLdapWatchdog process if there is any"
            PID=`ps aux |grep "createLdapWatchdog"|grep -v grep|awk '{print $2}'`
            echo "Pid:$PID"
            if [ -z "$PID" ]; then 
             echo "No createLdapWatchdog Pid"
            else
             kill $PID
             echo "Killed createLdapWatchdog process $PID"
            fi
            
            
	      # stop service and remove it from OS control
            chkconfig_out=`/sbin/chkconfig --del ldap`
            rc=$?
            smha_debug "/sbin/chkconfig --del ldap: output: ${chkconfig_out} : rc: $rc"
            stop_out=`/etc/init.d/ldap stop`
            rc=$?
            smha_debug "/etc/init.d/ldap stop: output: ${stop_out} : rc: $rc"
	      ;;
   esac
}

Action=${1}
position=${2}

smha_debug "conf_ldap.sh called with ${1} ${2}"
# when configuration is started, "conf" is invoked with "begin", then "end"
# when configuration is terminated, "unconf" is invoked with "begin", then "end"
case ${Action} in
    conf)
        if [ "$position" == "begin" ]
        then
            smha_debug "action=conf position = begin"
            if modify_service off  
            then
                mv $NEW_INITTAB $INITTAB      # rename new file to original file name
                smha_debug "conf modify_service off succeed"
                init q
            else
                rm -f $NEW_INITTAB $BK_INITTAB   # cleanup after error
                smha_debug "conf modify_service on failed"             
            fi            
        else
		# position == end
		smha_debug "action=conf position = end"
        fi
        RC=0
        ;;
    unconf)
        if [ "$position" == "end" ]
        then
           smha_debug "action=unconf position = end"
           if modify_service on 
           then
                mv $NEW_INITTAB $INITTAB      # rename new file to original file name
                smha_debug "unconf modify_service on succeed"
                init q
           else
                rm -f $NEW_INITTAB $BK_INITTAB   # cleanup after error 
                smha_debug "unconf modify_service on failed"            
           fi          
        else
		# position == begin
		smha_debug "action=unconf position = begin"
        fi
        RC=0
        ;;    
    upgrade) 
         smha_debug "action=upgrade"
	     if modify_service off  
          then
            mv $NEW_INITTAB $INITTAB      # rename new file to original file name
            smha_debug "upgrade modify_service off succeed"
            init q
          else
            rm -f $NEW_INITTAB $BK_INITTAB   # cleanup after error
            smha_debug "upgrade modify_service off failed"             
         fi          
	     RC=0
	;;

esac
smha_debug "conf_ldap.sh exiting"
exit $RC

