#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# This script is used by High Availability for 
# invocation as part of HA configuration or termination.
# The script is called to start, end ldap server or return ldap
# server status on FSM.
# This script should be put in /opt/ibm/director/ha/plugins/tsa/


PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

EXPLOITER_COMMON_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh
SLAPD_BIN="/usr/lib/openldap/slapd"
HA_LOG="/opt/ibm/director/ha/logs/smha.log"

if [ -f $EXPLOITER_COMMON_CODE ]; then
    source $EXPLOITER_COMMON_CODE
else
    echo "$PROGNAME:Exploiter common code not found."
    exit 1
fi
#############################################################

RC=$TSA_UNKNOWN

Action=${1}

case ${Action} in
start)
    smha_record_tsa_action $Action LDAP "-"
    smha_debug "FSM (start) Check status of ldap using /etc/init.d/ldap status" 
    /etc/init.d/ldap status >> $HA_LOG

    smha_debug "FSM Check for slapd process running before starting ldap"  
    ps -ef | grep slapd | grep -v "grep" | grep "/usr/lib/openldap/slapd" > /dev/null
    LDAP_ACTIVE=$?
    if [ $LDAP_ACTIVE -eq 0 ]; then
      smha_debug "FSM ldap slapd process is running so kill slapd"  
      killall slapd  
    else
       smha_debug "FSM ldap is NOT running"  
   fi
   smha_debug "FSM start ldap "  
   /etc/init.d/ldap start >> $HA_LOG
   smha_debug "FSM Check ldap server to verify it is running after start"
   /etc/init.d/ldap status >> $HA_LOG
   ps -ef | grep slapd | grep -v "grep" | grep "/usr/lib/openldap/slapd" > /dev/null
   LDAP_ACTIVE=$?
   if [ $LDAP_ACTIVE -eq 0 ]; then
    smha_debug "FSM ldap is now running!"    
   else
    smha_debug "FSM ldap is NOT running - There is a problem. Check slapd process."  
  fi
  smha_debug "FSM dump the slapd process"  
  ps -ef | grep slapd >> $HA_LOG
    ;;

stop)
    smha_record_tsa_action $Action LDAP "-"
    #normally, you would call your stop process here
    #e.g. /etc/init.d/mydaemon stop
    # you may want to perform some sanity checking here to make sure
    # the process really died.  Especially in the case if the process
    # contains files in shared storage which needs to be unmounted
    # in the case of a manual failover.
    
    smha_debug "FSM (stop) ldap server stopping"
    /etc/init.d/ldap stop >> $HA_LOG
    smha_debug "FSM ldap server stop status"
    /etc/init.d/ldap status >> $HA_LOG
    ;;

status)    
    #normally, you would call your status process here
    #e.g. /etc/init.d/mydaemon status
    #statusrc=$?
    /etc/init.d/ldap status >> $HA_LOG

    if [ $? == 0 ]; then
        RC=${TSA_ONLINE}
    else
        RC=${TSA_OFFLINE}
    fi
    echo $RC
    ;;
esac
#debug_tsa "$PROGNAME $Action: rc=$RC"
smha_record_tsa_action $Action LDAP $RC
exit $RC