#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# This script should ship as /opt/ibm/director/ha/plugins/conf/datamgmt.sh

#include common exploiter code
EXPLOITER_COMMON_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh
FRM_PROP=/opt/ibm/director/lwi/conf/overrides/frm.properties

if [ -f $EXPLOITER_COMMON_CODE ]; then
    source $EXPLOITER_COMMON_CODE
else
    echo "$PROGNAME:Exploiter common code not found."
    exit 1
fi

# The following function obtains the data management IP addresses from
# /opt/ibm/director/lwi/conf/overrides/frm.properties
#
# N.B. at the point this script is called, this bind mount point has not
# yet been unmounted at the beginning of HA configuration, or it has been
# remounted after HA unconfigure is complete.

get_datamgmtIP () {
    # look for the DM_ULA_ETH0 and DM_ULA_ETH1 properties. One or both may
    # be present
	
	#Expected format is x:x:x:x:x:x:x:x
    DATAMGMT_IP0_ADDRESS=`sed '/^\#/d' "$FRM_PROP" | grep 'DM_ULA_ETH0'  | tail -n 1 | cut -d "=" -f2- | sed "s/\\\\\:/:/g" | sed 's/^[[:space:]]*//;s/[[:space:]]*$//' | cut -d "/" -f1`
    DATAMGMT_IP1_ADDRESS=`sed '/^\#/d' "$FRM_PROP" | grep 'DM_ULA_ETH1'  | tail -n 1 | cut -d "=" -f2- | sed "s/\\\\\:/:/g" | sed 's/^[[:space:]]*//;s/[[:space:]]*$//' | cut -d "/" -f1`
			
    if [ -z "$DATAMGMT_IP0_ADDRESS" -a -z "$DATAMGMT_IP1_ADDRESS" ]; then
        smha_debug "get_datamgmtIP: Failed to extract data management IP addresses."
        smha_sys_log "get_datamgmtIP: Failed to extract data management IP addresses."
        exit 99
    else
        smha_debug "get_datamgmtIP: IP0=$DATAMGMT_IP0_ADDRESS IP1=$DATAMGMT_IP1_ADDRESS"
    fi
}

# Following function saves the data management IP addresses to be used later in
# the HA configuration process.

save_datamgmtIP () {

    if [ -n "$DATAMGMT_IP0_ADDRESS" ]; then
        echo $DATAMGMT_IP0_ADDRESS >  /tmp/__datamgmtSavedIP0__
    fi

    if [ -n "$DATAMGMT_IP1_ADDRESS" ]; then
        echo $DATAMGMT_IP1_ADDRESS >  /tmp/__datamgmtSavedIP1__
    fi
}


###############  Main  ################

Action=${1}
position=${2}
    
smha_debug "datamgmt.sh called with ${1} ${2}"
# when configuration is started, "conf" is invoked with "begin", then "end"
# when configuration is terminated, "unconf" is invoked with "begin", then "end"


case ${Action} in

    conf)
        if [ "$position" == "begin" ] 
        then
            get_datamgmtIP
            save_datamgmtIP

            # at this point the data management ULA must be removed from the network interface to which
            # it is assigned

            if [ -n "$DATAMGMT_IP0_ADDRESS" ]; then
                smha_debug "Executing: /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth0 $DATAMGMT_IP0_ADDRESS/64 remove"
                /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth0 $DATAMGMT_IP0_ADDRESS/64 remove
            fi

            if [ -n "$DATAMGMT_IP1_ADDRESS" ]; then
                smha_debug "Executing: /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth1 $DATAMGMT_IP1_ADDRESS/64 remove"
                /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth1 $DATAMGMT_IP1_ADDRESS/64 remove
            fi
        fi
    ;;

    unconf)
        if [ "$position" == "end" ] 
        then
            get_datamgmtIP

            # at this point the data management ULA must be added to the network interface to which
            # it is assigned

            if [ -n "$DATAMGMT_IP0_ADDRESS" ]; then
                smha_debug "Executing: /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth0 $DATAMGMT_IP0_ADDRESS/64 add"
                /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth0 $DATAMGMT_IP0_ADDRESS/64 add
            fi

            if [ -n "$DATAMGMT_IP1_ADDRESS" ]; then
                smha_debug "Executing: /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth1 $DATAMGMT_IP1_ADDRESS/64 add"
                /opt/ibm/director/lwi/runtime/itme/addremoveulaip.sh eth1 $DATAMGMT_IP1_ADDRESS/64 add
            fi
        fi
  ;;
  
esac

