#!/bin/bash

#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2010 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#

EXPLOITER_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh

STANDBY_RG=frmstandby-rg
STANDBY_RSRC_NAME=frmstandby
STANDBY_SCRIPTS=/opt/ibm/director/standby/bin
STANDBY_MONITOR_SCRIPT=smstandby.sh
STANDBY_MON_CMD_PERIOD=60
STANDBY_MON_CMD_TIMEOUT=20
STANDBY_MON_START_TIMEOUT=300
STANDBY_MON_STOP_TIMEOUT=300

if [ -f $EXPLOITER_CODE ]; then
    source $EXPLOITER_CODE
else
    echo "smfrmstandby.sh:Exploiter code not found."
    exit 1
fi
#############################################################

mk_standby () {
    smha_debug "Entered mk_standby"
    
    for i in {1..100}
    do

        smha_debug "Create the FRM standby resources"
        mkrsrc IBM.Application \
          Name="$STANDBY_RSRC_NAME" \
          StartCommand="$STANDBY_SCRIPTS/$STANDBY_MONITOR_SCRIPT start" \
          StopCommand="$STANDBY_SCRIPTS/$STANDBY_MONITOR_SCRIPT stop" \
          MonitorCommand="$STANDBY_SCRIPTS/$STANDBY_MONITOR_SCRIPT status" \
          MonitorCommandPeriod=$STANDBY_MON_CMD_PERIOD \
          MonitorCommandTimeout=$STANDBY_MON_CMD_TIMEOUT \
          NodeNameList="{'$LocalNode','$RemoteNode'}" \
          StartCommandTimeout=$STANDBY_MON_START_TIMEOUT \
          StopCommandTimeout=$STANDBY_MON_STOP_TIMEOUT \
          UserName="root" \
          ResourceType=1

        if [ $? -eq 0 ]; then
            smha_debug "Standby resource created"
            smha_debug "Exited mk_standby"
            return 0
        else
            smha_debug "Retry making rsrc $STANDBY_RSRC_NAME"
            sleep 5       
        fi
        
    done
    
    smha_debug "Exited mk_standby"
    
    return 1
}


crt_standby_rsrc () {
    smha_debug "Entered crt_standby_rsrc"

    # Only create the resource if it doesn't already exist
    lsrg -A d -g $STANDBY_RG &>/dev/null
    if [ $? -eq 6 ]; then
        
        mk_standby
        
        smha_debug "FRM standby resources created"
        # create the resource group
        mkrg -l none $STANDBY_RG
        
        # add to resource group
		smha_add_to_rg $STANDBY_RSRC_NAME IBM.Application $STANDBY_RG O F

    else
        smha_debug "FRM standby resources already created"   
    fi
    
    smha_debug "Exited crt_standby_rsrc"
}

crt_standby_rel () {
    smha_debug "Entered crt_standby_rel"

    # FRM Standby CANNOT be started at same time as Director
    smha_make_relationship AntiCollocated IBM.Application:$STANDBY_RSRC_NAME \
        IBM.Application:${SMHA_DIRECTOR_RSRC_NAME} $STANDBY_RSRC_NAME-anticoll-${SMHA_DIRECTOR_RSRC_NAME}
        
    smha_make_relationship AntiCollocated IBM.Application:${SMHA_DIRECTOR_RSRC_NAME} \
        IBM.Application:$STANDBY_RSRC_NAME ${SMHA_DIRECTOR_RSRC_NAME}-anticoll-$STANDBY_RSRC_NAME
        
    smha_debug "Exited crt_standby_rel"
}

rmv_standby_rel () {
    smha_debug "Entered rmv_standby_rel"
    rmrel -S IBM.Application:$STANDBY_RSRC_NAME
    rmrel -S IBM.Application:${SMHA_DIRECTOR_RSRC_NAME}
    smha_debug "Exited rmv_standby_rel"
}

rmv_standby_rg () {
    smha_debug "Entered rmv_standby_rg"
    rmrg $STANDBY_RG
    rmrsrc -s "Name == '$STANDBY_RSRC_NAME'" IBM.Application Force=1
    smha_debug "Exited rmv_standby_rg"
}


#### Main ########

LocalNode=$(smha_get_local_rsct_node_name)
RemoteNode=$(smha_get_remote_rsct_node_name)

method=$1

case ${method} in
create)
    crt_standby_rsrc
    crt_standby_rel
    ;;        
delete)
    smha_debug "Delete for FRM standby called."
    rmv_standby_rel
    rmv_standby_rg
    ;;        
*)
    smha_fatal_error "Bad parameter to smfrmstandby.sh"
    ;;        
esac

